package jdk.internal.org.objectweb.asm.commons;

import jdk.internal.org.objectweb.asm.MethodVisitor;
import jdk.internal.org.objectweb.asm.FieldVisitor;
import jdk.internal.org.objectweb.asm.TypePath;
import jdk.internal.org.objectweb.asm.AnnotationVisitor;
import jdk.internal.org.objectweb.asm.ClassVisitor;

public class RemappingClassAdapter extends ClassVisitor
{
    protected final Remapper remapper;
    protected String className;
    
    public RemappingClassAdapter(ClassVisitor classVisitor, Remapper remapper) {
        this(327680, classVisitor, remapper);
    }
    
    protected RemappingClassAdapter(int api, ClassVisitor classVisitor, Remapper remapper) {
        super(api, classVisitor);
        this.remapper = remapper;
    }
    
    @Override
    public void visit(int version, int access, String className, String s, String s2, String[] array) {
        this.className = className;
        super.visit(version, access, this.remapper.mapType(className), this.remapper.mapSignature(s, false), this.remapper.mapType(s2), (array == null) ? null : this.remapper.mapTypes(array));
    }
    
    @Override
    public AnnotationVisitor visitAnnotation(String s, boolean visible) {
        AnnotationVisitor visitAnnotation = super.visitAnnotation(this.remapper.mapDesc(s), visible);
        return (visitAnnotation == null) ? null : this.createRemappingAnnotationAdapter(visitAnnotation);
    }
    
    @Override
    public AnnotationVisitor visitTypeAnnotation(int typeRef, TypePath typePath, String s, boolean visible) {
        AnnotationVisitor visitTypeAnnotation = super.visitTypeAnnotation(typeRef, typePath, this.remapper.mapDesc(s), visible);
        return (visitTypeAnnotation == null) ? null : this.createRemappingAnnotationAdapter(visitTypeAnnotation);
    }
    
    @Override
    public FieldVisitor visitField(int access, String s, String s2, String s3, Object o) {
        FieldVisitor visitField = super.visitField(access, this.remapper.mapFieldName(this.className, s, s2), this.remapper.mapDesc(s2), this.remapper.mapSignature(s3, true), this.remapper.mapValue(o));
        return (visitField == null) ? null : this.createRemappingFieldAdapter(visitField);
    }
    
    @Override
    public MethodVisitor visitMethod(int access, String s, String s2, String s3, String[] array) {
        String mapMethodDesc = this.remapper.mapMethodDesc(s2);
        MethodVisitor visitMethod = super.visitMethod(access, this.remapper.mapMethodName(this.className, s, s2), mapMethodDesc, this.remapper.mapSignature(s3, false), (array == null) ? null : this.remapper.mapTypes(array));
        return (visitMethod == null) ? null : this.createRemappingMethodAdapter(access, mapMethodDesc, visitMethod);
    }
    
    @Override
    public void visitInnerClass(String s, String s2, String innerName, int access) {
        super.visitInnerClass(this.remapper.mapType(s), (s2 == null) ? null : this.remapper.mapType(s2), innerName, access);
    }
    
    @Override
    public void visitOuterClass(String s, String s2, String s3) {
        super.visitOuterClass(this.remapper.mapType(s), (s2 == null) ? null : this.remapper.mapMethodName(s, s2, s3), (s3 == null) ? null : this.remapper.mapMethodDesc(s3));
    }
    
    protected FieldVisitor createRemappingFieldAdapter(FieldVisitor fieldVisitor) {
        return new RemappingFieldAdapter(fieldVisitor, this.remapper);
    }
    
    protected MethodVisitor createRemappingMethodAdapter(int n, String s, MethodVisitor methodVisitor) {
        return new RemappingMethodAdapter(n, s, methodVisitor, this.remapper);
    }
    
    protected AnnotationVisitor createRemappingAnnotationAdapter(AnnotationVisitor annotationVisitor) {
        return new RemappingAnnotationAdapter(annotationVisitor, this.remapper);
    }
}
